@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-ui.h,v 1.11 2006/05/29 10:14:28 paolo Exp $
+ * $Id: gedit-ui.h,v 1.13 2006/06/18 09:05:20 paolo Exp $
  */
 
 #ifndef __GEDIT_UI_H__
@@ -64,8 +64,6 @@ static const GtkActionEntry gedit_always_sensitive_menu_entries[] =
 	  N_("Open a file"), G_CALLBACK (_gedit_cmd_file_open) },
 	{ "FileOpenURI", NULL, N_("Open _Location..."), "<control>L",
 	  N_("Open a file from a specified location"), G_CALLBACK (_gedit_cmd_file_open_uri) },
-	{ "FilePageSetup", NULL, N_("Page Set_up..."), NULL,
-	  N_("Setup the page settings"), G_CALLBACK (_gedit_cmd_file_page_setup) },
 	
 	/* Edit menu */
 	{ "EditPreferences", GTK_STOCK_PREFERENCES, N_("Pr_eferences"), NULL,
@@ -87,6 +85,8 @@ static const GtkActionEntry gedit_menu_entries[] =
 	  N_("Save the current file with a different name"), G_CALLBACK (_gedit_cmd_file_save_as) },
 	{ "FileRevert", GTK_STOCK_REVERT_TO_SAVED, NULL, NULL,
 	  N_("Revert to a saved version of the file"), G_CALLBACK (_gedit_cmd_file_revert) },
+	{ "FilePageSetup", NULL, N_("Page Set_up..."), NULL,
+	  N_("Setup the page settings"), G_CALLBACK (_gedit_cmd_file_page_setup) },
 	{ "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, N_("Print Previe_w"),"<control><shift>P",
 	  N_("Print preview"), G_CALLBACK (_gedit_cmd_file_print_preview) },
 	 { "FilePrint", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
@@ -126,6 +126,8 @@ static const GtkActionEntry gedit_menu_entries[] =
 	  N_("Clear highlighting of search matches"), G_CALLBACK (_gedit_cmd_search_clear_highlight) },
 	{ "SearchGoToLine", GTK_STOCK_JUMP_TO, N_("Go to _Line..."), "<control>I",
 	  N_("Go to a specific line"), G_CALLBACK (_gedit_cmd_search_goto_line) },
+	{ "SearchInteractiveSearch", GTK_STOCK_FIND, N_("Interactive search..."), "<control>K",
+	  N_("Interactively search for text"), G_CALLBACK (_gedit_cmd_search_interactive_search) },
 
 	/* Documents menu */
 	{ "FileSaveAll", GTK_STOCK_SAVE, N_("_Save All"), "<shift><control>L",
